SUBROUTINE t_analytic()
USE jm_constants
USE jm_global, ONLY: k, kpr, scheme, a, b, eps, epspr, l, scr, v0, shift

! Computes analytical value of the phase shift for the square-well potential.

IMPLICIT none

REAL(KIND=dp) :: x1(2,2), x2(2,2), y1(2,2), y2(2,2), il1(2,2), il2(2,2)
REAL(KIND=dp) :: ld, ka, kb, k1a, k1b
REAL(KIND=dp) :: sphbess, sphneum, sphbess_prim, sphneum_prim
REAL(KIND=dp) :: tan_delta_analytic, delta_analytic
REAL(KIND=dp) :: jka, jk1a, jpka, jkb, jpkb, jk1b, jpk1b, jpk1a
REAL(KIND=dp) :: nk1a, npk1a, nk1b, nkb, npk1b, npkb


IF (scheme == 'relativistic') THEN
  
  CALL formy (y1, b, k,   eps)
  CALL formx (x1, b, kpr, epspr)
  CALL formy (y2, a, kpr, epspr)
  CALL formx (x2, a, k,   eps)
  
  il1 = matmul(y2,x2)
  il2 = matmul(x1,il1)
  il1 = matmul(y1,il2)
  
  tan_delta_analytic = il1(2,1) / il1(1,1)
  
ELSE   ! Non-relativic case not finished yet - only for testing purposes

!  IF (v0 < 0) RETURN
  
  ka  = k * a    ; kb  = k * b
  k1a = kpr * a  ; k1b = kpr * b

  ld = DBLE(l)
  
  jka  = sphbess(ld,ka)   ; jpka  = sphbess_prim(ld,ka)
  nk1a = sphneum(ld,k1a)  ; npk1a = sphneum_prim(ld,k1a)
  jk1a = sphbess(ld,k1a)  ; jpk1a = sphbess_prim(ld,k1a) 
  jk1b = sphbess(ld,k1b)  ; jpkb  = sphbess_prim(ld,kb)
  jkb  = sphbess(ld,kb)   ; npk1b = sphneum_prim(ld,k1b)
  nkb  = sphneum(ld,kb)   ; jpk1b = sphbess_prim(ld,k1b)
  nk1b = sphneum(ld,k1b)  ; npkb  = sphneum_prim(ld,kb)

  tan_delta_analytic =                                                    &
  (jpkb*(jk1b*jpka*nk1a+jka*jpk1a*nk1b-jk1a*jpka*nk1b-jk1b*jka*npk1a) +   &
  jkb*(-jpk1b*jpka*nk1a+jka*jpk1b*npk1a-jka*jpk1a*npk1b+jk1a*jpka*npk1b)) &
  / (jpk1b*jpka*nk1a*nkb-jka*jpk1b*nkb*npk1a+jka*jpk1a*nkb*npk1b-         &
  jk1a*jpka*nkb*npk1b-jk1b*jpka*nk1a*npkb-jka*jpk1a*nk1b*npkb+            &
  jk1a*jpka*nk1b*npkb+jk1b*jka*npk1a*npkb)

ENDIF

delta_analytic = atan(tan_delta_analytic)

IF (shift .AND. delta_analytic < zero) delta_analytic = delta_analytic + pi

IF (scr) WRITE(*,*)
IF (scr) WRITE(*,'(A,F20.16)') 'delta_analytic=',delta_analytic

WRITE(2,'(A19,F20.15)') '; ANALYTICAL RESULT'
WRITE(2,'(A10,F20.15)')  '; delta = ',delta_analytic
WRITE(2,'(A1)') ';'

RETURN
END SUBROUTINE t_analytic

!***********************************************************************

SUBROUTINE formx(x,r,k,eps)
USE jm_constants
USE jm_global, ONLY: l, kappa

IMPLICIT none

REAL(KIND=dp), INTENT(in)  :: r, k, eps
REAL(KIND=dp), INTENT(out) :: x(2,2)
REAL(KIND=dp)              :: kr, ld
REAL(KIND=dp)              :: sphneum, sphbess


kr = k * r
ld = DBLE(l)

x(1,1) =  sphbess(ld, kr)
x(1,2) = -sphneum(ld, kr)

IF (kappa > 0) THEN
  x(2,1) =  eps * sphbess(ld-one, kr)
  x(2,2) = -eps * sphneum(ld-one, kr)
ELSE
  x(2,1) = -eps * sphbess(ld+one, kr)
  x(2,2) =  eps * sphneum(ld+one, kr)
ENDIF

RETURN
END SUBROUTINE formx

!***********************************************************************

SUBROUTINE formy(y,r,k,eps)
USE jm_constants
USE jm_global, ONLY: l, kappa

IMPLICIT none
REAL(KIND=dp), INTENT(in)  :: r, k, eps
REAL(KIND=dp), INTENT(out) :: y(2,2)

REAL(KIND=dp)              :: kr, ld
REAL(KIND=dp)              :: sphneum, sphbess


kr = k * r
ld = DBLE(l)

y(1,2) = sphneum(ld, kr)
y(2,2) = sphbess(ld, kr)

IF (kappa > 0) THEN
  y(1,1) = -eps * sphneum(ld-one, kr)
  y(2,1) = -eps * sphbess(ld-one, kr)
ELSE
  y(1,1) =  eps * sphneum(ld+one, kr)
  y(2,1) =  eps * sphbess(ld+one, kr)
ENDIF

RETURN
END SUBROUTINE formy

!***********************************************************************