FUNCTION fi(n,r)
USE jm_constants
USE jm_global, ONLY: l, lambda, basis

! Forms the basis function.

IMPLICIT none

INTEGER, INTENT(IN)        :: n
REAL(KIND=dp), INTENT(IN)  :: r
REAL(KIND=dp)              :: lr, lr2
REAL(KIND=dp)              :: fi
REAL(KIND=dp)              :: laguerre

lr = lambda * r 

IF (basis == 'laguerre') THEN   ! Laguerre basis

  fi = lr**(l+1) * exp(-lr/two) * laguerre(n,2*l+one,lr)
  
ELSE    ! Gaussian basis

  lr2 = lr*lr
  fi  = lr**(l+1) * exp(-lr2/two) * laguerre(n,l+half,lr2)
  
ENDIF

RETURN
END FUNCTION fi

!***********************************************************************

FUNCTION psi(n,r)
USE jm_constants
USE jm_global, ONLY: l, lambda, basis, kappa

! Forms the basis function.

IMPLICIT none

INTEGER, INTENT(IN)       :: n
REAL(KIND=dp), INTENT(IN) :: r
REAL(KIND=dp)             :: lr, lr2
REAL(KIND=dp)             :: fi, laguerre
REAL(KIND=dp)             :: psi


lr = lambda * r

IF (basis == 'laguerre') THEN
  
  psi = kappa / r * fi(n,r) + exp(-lr/two) *        (                 &
      lambda**(l+1) * (l+1) * r**l * laguerre(n,2*l+one,lr) -         &
      lr**(l+1) * lambda/two * laguerre(n,2*l+one,lr) -               &
      lr**(l+1) * (laguerre(n,2*l+two,lr)-laguerre(n,2*l+one,lr)) )
  
ELSE
  
  lr2 = lr * lr
  psi = kappa / r * fi(n,r) + exp(-lr2/two) *        (                &
      lambda**(l+1) * (l+1) * r**l * laguerre(n,l+half,lr2)-          &
      lr**(l+1) * lambda**2 * r * laguerre(n,l+half,lr2) -            &
      lr**(l+1) * (laguerre(n,l+1.5_dp,lr2)-laguerre(n,l+half,lr2)))
  
ENDIF

RETURN
END FUNCTION psi

!***********************************************************************

FUNCTION s(n)
USE jm_constants
USE jm_global, ONLY: l, lambda, scheme, nend, k, basis, e, estart

! Computes the coefficients ot the sine-like solution.

IMPLICIT none

INTEGER, INTENT(IN)              :: n
INTEGER                          :: i, tend
REAL(KIND=dp)                    :: gamma, gegen, laguerre, fact, jm
REAL(KIND=dp)                    :: sint, cost, eta
REAL(KIND=dp), ALLOCATABLE, SAVE :: t_s(:)
REAL(KIND=dp)                    :: s
CHARACTER (LEN=16)               :: tmp_scheme


IF (.NOT. allocated(t_s) .OR. e/=estart) THEN
  IF (.NOT. allocated(t_s)) ALLOCATE(t_s(0:nend))
  tend = nend
  IF (nend<3) tend = 5
  sint = k / ( k*k/lambda + lambda/four)
  cost = (k**2 / lambda**2 - 0.25_dp)/(k**2 / lambda**2 + 0.25_dp)
  eta = k / lambda
  IF (basis == 'laguerre') THEN
    t_s(0) = 2**l * gamma(l+one)/fact(2*l+1)*sint**(l+1)*gegen(0,l+1,cost)
    t_s(1) = 2**l * gamma(l+one)/fact(2*l+2)*sint**(l+1)*gegen(1,l+1,cost)
  ELSE
    t_s(0) = sqrt(twopi)  / gamma(l+1.5_dp) *  &
       exp(-eta**2 / two) * laguerre(0,l+half,eta**2) * eta**(l+1)
    t_s(1) =-sqrt(twopi)  / gamma(l+2.5_dp) *  &
       exp(-eta**2 / two) * laguerre(1,l+half,eta**2) * eta**(l+1)
  ENDIF
  DO i = 2, tend
    IF (scheme == 'relativistic' ) THEN
      tmp_scheme = scheme
      scheme = 'non-relativistic'
      t_s(i) = -( jm(i-1,i-2)*t_s(i-2) + jm(i-1,i-1)*t_s(i-1) ) / jm(i-1,i)
      scheme = tmp_scheme
    ELSE
      t_s(i) = -( jm(i-1,i-2)*t_s(i-2) + jm(i-1,i-1)*t_s(i-1) ) / jm(i-1,i)
    ENDIF
  ENDDO
ENDIF

s = t_s(n)

RETURN
END FUNCTION s

!***********************************************************************

FUNCTION c(n)
USE jm_constants
USE jm_global, ONLY: scheme, nend, k, lambda, basis, l, e, estart

! Computes the coefficients ot the cosine-like solution.

IMPLICIT none

INTEGER, INTENT(IN)              :: n
INTEGER                          :: i, tend
REAL(KIND=dp)                    :: c
REAL(KIND=dp)                    :: fact, gamma, hyperg21, hyperg11, jm
REAL(KIND=dp)                    :: sint, sin2t, eta
REAL(KIND=dp), ALLOCATABLE, SAVE :: t_c(:)
CHARACTER (LEN=16)               :: tmp_scheme


IF (.NOT. allocated(t_c) .OR. e/=estart) THEN
  IF (.NOT. allocated(t_c)) ALLOCATE(t_c(0:nend))
  tend = nend
  IF (tend<3) tend = 5
  sint = k / ( k*k/lambda + lambda/four)
  sin2t = lambda**2 / (four * k**2 + lambda**2)
  eta = k / lambda
  IF (basis == 'laguerre') THEN
    t_c(0) = -(2)**l / sint**l / fact(2*l+1) *                          &
      hyperg21(-two*l-one, one, half-l, sin2t)/sqrt(pi)*gamma(l+half)
    t_c(1) = -(2)**l / sint**l / fact(2+2*l) *                          &
      hyperg21(-two*l-two, two, half-l, sin2t)/sqrt(pi)*gamma(l+half)
  ELSE
    t_c(0) = sqrt(two/pi)*gamma(l+half) / gamma(l+1.5_dp)                &
      * exp(-eta**2/two) * eta**(-l) * hyperg11(-l-half,half-l,eta**2)
    t_c(1) = sqrt(two/pi) * gamma(l+half) * (-1) / gamma(l+2.5_dp)       &
      * exp(-eta**2/two) * eta**(-l) * hyperg11(-l-1.5_dp,half-l,eta**2)
  ENDIF
  DO i = 2, tend     ! three-term recursion relation
    IF (scheme == 'relativistic' ) THEN
      tmp_scheme = scheme
      scheme = 'nonrelativistic'
      t_c(i) = -( jm(i-1,i-2)*t_c(i-2) + jm(i-1,i-1)*t_c(i-1) ) / jm(i-1,i)
      scheme = tmp_scheme
    ELSE
      t_c(i) = -( jm(i-1,i-2)*t_c(i-2) + jm(i-1,i-1)*t_c(i-1) ) / jm(i-1,i)
    ENDIF
  ENDDO
ENDIF

c = t_c(n)

RETURN
END FUNCTION c

!***********************************************************************

FUNCTION fifi(m,n)
USE jm_constants
USE jm_global, ONLY: basis, l, lambda

IMPLICIT none

INTEGER, INTENT(IN) :: m, n
REAL(KIND=dp)       :: fifi
REAL(KIND=dp)       :: k_delta, lfact, lgamma


IF (basis == 'laguerre') THEN
  
  fifi = exp(lfact(n+2*l+1)-lfact(n)) / lambda *   &
   ( two*(n+l+1)*k_delta(m,n) - n*k_delta(m,n-1) - (n+2*l+2)*k_delta(m,n+1) )

ELSE
  
 fifi = exp(lgamma(n+l+1.5_dp)-lfact(n)) / two * k_delta(m,n)
  
ENDIF

RETURN
END FUNCTION fifi

!***********************************************************************

FUNCTION psipsi(m,n)
USE jm_constants
USE jm_global, ONLY: basis, l, lambda

IMPLICIT none

INTEGER, INTENT(IN) :: m, n
REAL(KIND=dp)       :: psipsi
REAL(KIND=dp)       :: k_delta, lfact, lgamma


IF (basis == 'laguerre') THEN
  
  psipsi = exp(lfact(n+2*l+1)-lfact(n)) / four *         &
     ( two*(n+l+1)*(2*lambda-1)*k_delta(m,n)             &
       + n*k_delta(m,n-1) + (n+2*l+2)*k_delta(m,n+1) )
  
ELSE
  
  psipsi = lambda**2 * exp(lgamma(n+l+1.5_dp) - lfact(n)) / two *  &
((2*n+l+1.5_dp)*k_delta(m,n) + n*k_delta(m,n-1) + (n+l+1.5_dp)*k_delta(m,n+1))
  
ENDIF

RETURN
END FUNCTION psipsi

!***********************************************************************
